@@ -2,8 +2,10 @@
 #[path = "tests.rs"]
 mod tests;
 
+use self::Normalization::*;
 use crate::directory::Directory;
 use crate::run::PathDependency;
+use std::cmp;
 use std::path::Path;
 
 #[derive(Copy, Clone)]
@@ -16,18 +18,47 @@ pub struct Context<'a> {
     pub path_dependencies: &'a [PathDependency],
 }
 
-pub fn trim<S: AsRef<[u8]>>(output: S) -> String {
-    let bytes = output.as_ref();
-    let mut normalized = String::from_utf8_lossy(bytes).into_owned();
+macro_rules! normalizations {
+    ($($name:ident,)*) => {
+        #[derive(PartialOrd, PartialEq, Copy, Clone)]
+        enum Normalization {
+            $($name,)*
+        }
 
-    let len = normalized.trim_end().len();
-    normalized.truncate(len);
+        impl Normalization {
+            const ALL: &'static [Self] = &[$($name),*];
+        }
 
-    if !normalized.is_empty() {
-        normalized.push('\n');
-    }
+        impl Default for Variations {
+            fn default() -> Self {
+                Variations {
+                    variations: [$(($name, String::new()).1),*],
+                }
+            }
+        }
+    };
+}
 
-    normalized
+normalizations! {
+    Basic,
+    StripCouldNotCompile,
+    StripCouldNotCompile2,
+    StripForMoreInformation,
+    StripForMoreInformation2,
+    TrimEnd,
+    RustLib,
+    TypeDirBackslash,
+    WorkspaceLines,
+    PathDependencies,
+    CargoRegistry,
+    ArrowOtherCrate,
+    RelativeToDir,
+    LinesOutsideInputFile,
+    Unindent,
+    AndOthers,
+    StripLongTypeNameFiles,
+    // New normalization steps are to be inserted here at the end so that any
+    // snapshots saved before your normalization change remain passing.
 }
 
 /// For a given compiler output, produces the set of saved outputs against which
@@ -44,31 +75,16 @@ pub fn trim<S: AsRef<[u8]>>(output: S) -> String {
 pub fn diagnostics(output: &str, context: Context) -> Variations {
     let output = output.replace("\r\n", "\n");
 
-    let variations = [
-        Basic,
-        StripCouldNotCompile,
-        StripCouldNotCompile2,
-        StripForMoreInformation,
-        StripForMoreInformation2,
-        TrimEnd,
-        RustLib,
-        TypeDirBackslash,
-        WorkspaceLines,
-        PathDependencies,
-        CargoRegistry,
-        ArrowOtherCrate,
-        RelativeToDir,
-        LinesOutsideInputFile,
-    ]
-    .iter()
-    .map(|normalization| apply(&output, *normalization, context))
-    .collect();
+    let mut result = Variations::default();
+    for (i, normalization) in Normalization::ALL.iter().enumerate() {
+        result.variations[i] = apply(&output, *normalization, context);
+    }
 
-    Variations { variations }
+    result
 }
 
 pub struct Variations {
-    variations: Vec<String>,
+    variations: [String; Normalization::ALL.len()],
 }
 
 impl Variations {
@@ -79,29 +95,30 @@ impl Variations {
     pub fn any<F: FnMut(&str) -> bool>(&self, mut f: F) -> bool {
         self.variations.iter().any(|stderr| f(stderr))
     }
+
+    pub fn concat(&mut self, other: &Self) {
+        for (this, other) in self.variations.iter_mut().zip(&other.variations) {
+            if !this.is_empty() && !other.is_empty() {
+                this.push('\n');
+            }
+            this.push_str(other);
+        }
+    }
 }
 
-#[derive(PartialOrd, PartialEq, Copy, Clone)]
-enum Normalization {
-    Basic,
-    StripCouldNotCompile,
-    StripCouldNotCompile2,
-    StripForMoreInformation,
-    StripForMoreInformation2,
-    TrimEnd,
-    RustLib,
-    TypeDirBackslash,
-    WorkspaceLines,
-    PathDependencies,
-    CargoRegistry,
-    ArrowOtherCrate,
-    RelativeToDir,
-    LinesOutsideInputFile,
-    // New normalization steps are to be inserted here at the end so that any
-    // snapshots saved before your normalization change remain passing.
-}
+pub fn trim<S: AsRef<[u8]>>(output: S) -> String {
+    let bytes = output.as_ref();
+    let mut normalized = String::from_utf8_lossy(bytes).into_owned();
 
-use self::Normalization::*;
+    let len = normalized.trim_end().len();
+    normalized.truncate(len);
+
+    if !normalized.is_empty() {
+        normalized.push('\n');
+    }
+
+    normalized
+}
 
 fn apply(original: &str, normalization: Normalization, context: Context) -> String {
     let mut normalized = String::new();
@@ -122,6 +139,10 @@ fn apply(original: &str, normalization: Normalization, context: Context) -> Stri
         }
     }
 
+    if normalization >= Unindent {
+        normalized = unindent(normalized);
+    }
+
     trim(normalized)
 }
 
@@ -265,9 +286,13 @@ impl<'a> Filter<'a> {
             }
             if self.normalization >= CargoRegistry && !other_crate {
                 if let Some(pos) = line.find("/registry/src/github.com-") {
-                    // ::: $CARGO/github.com-1ecc6299db9ec823/serde_json-1.0.64/src/de.rs:2584:8
-                    line.replace_range(line.find(prefix).unwrap() + 4..pos + 41, "$CARGO");
-                    other_crate = true;
+                    let start_cargo = line.find(prefix).unwrap() + 4;
+                    let end_cargo = pos + 41;
+                    if line.is_char_boundary(end_cargo) {
+                        // ::: $CARGO/github.com-1ecc6299db9ec823/serde_json-1.0.64/src/de.rs:2584:8
+                        line.replace_range(start_cargo..end_cargo, "$CARGO");
+                        other_crate = true;
+                    }
                 }
             }
             if other_crate && self.normalization >= WorkspaceLines {
@@ -335,6 +360,30 @@ impl<'a> Filter<'a> {
             }
         }
 
+        if self.normalization >= AndOthers {
+            let trim_start = line.trim_start();
+            if trim_start.starts_with("and ") && line.ends_with(" others") {
+                let indent = line.len() - trim_start.len();
+                let num_start = indent + "and ".len();
+                let num_end = line.len() - " others".len();
+                if num_start < num_end
+                    && line[num_start..num_end].bytes().all(|b| b.is_ascii_digit())
+                {
+                    line.replace_range(num_start..num_end, "$N");
+                }
+            }
+        }
+
+        if self.normalization >= StripLongTypeNameFiles {
+            let trimmed_line = line.trim_start();
+            let trimmed_line = trimmed_line
+                .strip_prefix("= note: ")
+                .unwrap_or(trimmed_line);
+            if trimmed_line.starts_with("the full type name has been written to") {
+                return None;
+            }
+        }
+
         line = line.replace(self.context.krate, "$CRATE");
         line = replace_case_insensitive(&line, &self.context.source_dir.to_string_lossy(), "$DIR/");
         line = replace_case_insensitive(
@@ -402,3 +451,121 @@ fn replace_case_insensitive(line: &str, pattern: &str, replacement: &str) -> Str
 
     replaced
 }
+
+#[derive(PartialEq)]
+enum IndentedLineKind {
+    // `error`
+    // `warning`
+    Heading,
+
+    // Contains max number of spaces that can be cut based on this line.
+    // `   --> foo` = 2
+    // `    | foo` = 3
+    // `   ::: foo` = 2
+    // `10  | foo` = 1
+    Code(usize),
+
+    // `note:`
+    // `...`
+    Note,
+
+    // Contains number of leading spaces.
+    Other(usize),
+}
+
+fn unindent(diag: String) -> String {
+    let mut normalized = String::new();
+    let mut lines = diag.lines();
+
+    while let Some(line) = lines.next() {
+        normalized.push_str(line);
+        normalized.push('\n');
+
+        if indented_line_kind(line) != IndentedLineKind::Heading {
+            continue;
+        }
+
+        let mut ahead = lines.clone();
+        let next_line = match ahead.next() {
+            Some(line) => line,
+            None => continue,
+        };
+
+        if let IndentedLineKind::Code(indent) = indented_line_kind(next_line) {
+            if next_line[indent + 1..].starts_with("--> ") {
+                let mut lines_in_block = 1;
+                let mut least_indent = indent;
+                while let Some(line) = ahead.next() {
+                    match indented_line_kind(line) {
+                        IndentedLineKind::Heading => break,
+                        IndentedLineKind::Code(indent) => {
+                            lines_in_block += 1;
+                            least_indent = cmp::min(least_indent, indent);
+                        }
+                        IndentedLineKind::Note => lines_in_block += 1,
+                        IndentedLineKind::Other(spaces) => {
+                            if spaces > 10 {
+                                lines_in_block += 1;
+                            } else {
+                                break;
+                            }
+                        }
+                    }
+                }
+                for _ in 0..lines_in_block {
+                    let line = lines.next().unwrap();
+                    if let IndentedLineKind::Code(_) | IndentedLineKind::Other(_) =
+                        indented_line_kind(line)
+                    {
+                        let space = line.find(' ').unwrap();
+                        normalized.push_str(&line[..space]);
+                        normalized.push_str(&line[space + least_indent..]);
+                    } else {
+                        normalized.push_str(line);
+                    }
+                    normalized.push('\n');
+                }
+            }
+        }
+    }
+
+    normalized
+}
+
+fn indented_line_kind(line: &str) -> IndentedLineKind {
+    if let Some(heading_len) = if line.starts_with("error") {
+        Some("error".len())
+    } else if line.starts_with("warning") {
+        Some("warning".len())
+    } else {
+        None
+    } {
+        if line[heading_len..].starts_with(&[':', '['][..]) {
+            return IndentedLineKind::Heading;
+        }
+    }
+
+    if line.starts_with("note:") || line == "..." {
+        return IndentedLineKind::Note;
+    }
+
+    let is_space = |b: &u8| *b == b' ';
+    if let Some(rest) = line.strip_prefix("... ") {
+        let spaces = rest.bytes().take_while(is_space).count();
+        return IndentedLineKind::Code(spaces);
+    }
+
+    let digits = line.bytes().take_while(u8::is_ascii_digit).count();
+    let spaces = line[digits..].bytes().take_while(|b| *b == b' ').count();
+    let rest = &line[digits + spaces..];
+    if spaces > 0
+        && (rest == "|"
+            || rest.starts_with("| ")
+            || digits == 0
+                && (rest.starts_with("--> ") || rest.starts_with("::: ") || rest.starts_with("= ")))
+    {
+        return IndentedLineKind::Code(spaces - 1);
+    }
+
+    IndentedLineKind::Other(if digits == 0 { spaces } else { 0 })
+}
