@@ -1,20 +1,31 @@
 use crate::directory::Directory;
 use crate::run::PathDependency;
+use std::path::Path;
 
 macro_rules! test_normalize {
-    ($name:ident $(DIR=$dir:literal)? $(WORKSPACE=$workspace:literal)? $original:literal $expected:literal) => {
+    (
+        $name:ident
+        $(DIR=$dir:literal)?
+        $(WORKSPACE=$workspace:literal)?
+        $(INPUT=$input:literal)?
+        $(TARGET=$target:literal)?
+        $original:literal
+        $expected:literal
+    ) => {
         #[test]
         fn $name() {
             let context = super::Context {
                 krate: "trybuild000",
+                input_file: Path::new({ "tests/ui/error.rs" $(; $input)? }),
                 source_dir: &Directory::new({ "/git/trybuild/test_suite" $(; $dir)? }),
                 workspace: &Directory::new({ "/git/trybuild" $(; $workspace)? }),
+                target_dir: &Directory::new({ "/git/trybuild/target" $(; $target)? }),
                 path_dependencies: &[PathDependency {
                     name: String::from("diesel"),
                     normalized_path: Directory::new("/home/user/documents/rust/diesel/diesel"),
                 }],
             };
-            let original = $original.to_owned().into_bytes();
+            let original = $original;
             let variations = super::diagnostics(original, context);
             let preferred = variations.preferred();
             let expected = $expected;
@@ -51,10 +62,12 @@ error[E0277]: the trait bound `QueryParams: serde::de::Deserialize<'de>` is not
    --> \\git\\trybuild\\test_suite\\tests\\ui\\error.rs:22:61
 " "
 error[E0277]: the trait bound `QueryParams: serde::de::Deserialize<'de>` is not satisfied
-   --> tests/ui/error.rs:22:61
+ --> tests/ui/error.rs:22:61
 "}
 
-test_normalize! {test_rust_lib "
+test_normalize! {test_rust_lib
+    INPUT="tests/ui/not-repeatable.rs"
+"
 error[E0599]: no method named `quote_into_iter` found for struct `std::net::Ipv4Addr` in the current scope
   --> /git/trybuild/test_suite/tests/ui/not-repeatable.rs:6:13
    |
@@ -70,21 +83,23 @@ error[E0599]: no method named `quote_into_iter` found for struct `std::net::Ipv4
    | doesn't satisfy `std::net::Ipv4Addr: quote::to_tokens::ToTokens`
 " "
 error[E0599]: no method named `quote_into_iter` found for struct `std::net::Ipv4Addr` in the current scope
-  --> tests/ui/not-repeatable.rs:6:13
-   |
-6  |     let _ = quote! { #(#ip)* };
-   |             ^^^^^^^^^^^^^^^^^^ method not found in `std::net::Ipv4Addr`
-   |
-  ::: $RUST/src/libstd/net/ip.rs
-  ::: $RUST/std/src/net/ip.rs
-   |
-   | pub struct Ipv4Addr {
-   | -------------------
-   | |
-   | doesn't satisfy `std::net::Ipv4Addr: quote::to_tokens::ToTokens`
+ --> tests/ui/not-repeatable.rs:6:13
+  |
+6 |     let _ = quote! { #(#ip)* };
+  |             ^^^^^^^^^^^^^^^^^^ method not found in `std::net::Ipv4Addr`
+  |
+ ::: $RUST/src/libstd/net/ip.rs
+ ::: $RUST/std/src/net/ip.rs
+  |
+  | pub struct Ipv4Addr {
+  | -------------------
+  | |
+  | doesn't satisfy `std::net::Ipv4Addr: quote::to_tokens::ToTokens`
 "}
 
-test_normalize! {test_type_dir_backslash "
+test_normalize! {test_type_dir_backslash
+    INPUT="tests/ui/compile-fail-3.rs"
+"
 error[E0277]: `*mut _` cannot be shared between threads safely
    --> /git/trybuild/test_suite/tests/ui/compile-fail-3.rs:7:5
     |
@@ -96,14 +111,14 @@ error[E0277]: `*mut _` cannot be shared between threads safely
     = note: required because it appears within the type `[closure@/git/trybuild/test_suite/ui/compile-fail-3.rs:7:19: 9:6 x:&*mut _]`
 " "
 error[E0277]: `*mut _` cannot be shared between threads safely
-   --> tests/ui/compile-fail-3.rs:7:5
-    |
-7   |     thread::spawn(|| {
-    |     ^^^^^^^^^^^^^ `*mut _` cannot be shared between threads safely
-    |
-    = help: the trait `std::marker::Sync` is not implemented for `*mut _`
-    = note: required because of the requirements on the impl of `std::marker::Send` for `&*mut _`
-    = note: required because it appears within the type `[closure@$DIR/ui/compile-fail-3.rs:7:19: 9:6 x:&*mut _]`
+ --> tests/ui/compile-fail-3.rs:7:5
+  |
+7 |     thread::spawn(|| {
+  |     ^^^^^^^^^^^^^ `*mut _` cannot be shared between threads safely
+  |
+  = help: the trait `std::marker::Sync` is not implemented for `*mut _`
+  = note: required because of the requirements on the impl of `std::marker::Send` for `&*mut _`
+  = note: required because it appears within the type `[closure@$DIR/ui/compile-fail-3.rs:7:19: 9:6 x:&*mut _]`
 "}
 
 test_normalize! {test_strip_path_dependencies "
@@ -152,15 +167,15 @@ For more information about this error, try `rustc --explain E0277`.
 error: could not compile `testing` due to previous error
 " "
 error[E0277]: the trait bound `Thread: serde::de::Deserialize<'_>` is not satisfied
-    --> src/main.rs:2:36
-     |
-2    |     let _ = serde_json::from_str::<std::thread::Thread>(\"???\");
-     |                                    ^^^^^^^^^^^^^^^^^^^ the trait `serde::de::Deserialize<'_>` is not implemented for `Thread`
-     |
-    ::: $CARGO/serde_json-1.0.64/src/de.rs
-     |
-     |     T: de::Deserialize<'a>,
-     |        ------------------- required by this bound in `serde_json::from_str`
+ --> src/main.rs:2:36
+  |
+2 |     let _ = serde_json::from_str::<std::thread::Thread>(\"???\");
+  |                                    ^^^^^^^^^^^^^^^^^^^ the trait `serde::de::Deserialize<'_>` is not implemented for `Thread`
+  |
+ ::: $CARGO/serde_json-1.0.64/src/de.rs
+  |
+  |     T: de::Deserialize<'a>,
+  |        ------------------- required by this bound in `serde_json::from_str`
 "}
 
 test_normalize! {test_traits_must_be_implemented "
@@ -207,46 +222,46 @@ note: the following traits must be implemented
     = note: this error originates in the macro `anyhow` (in Nightly builds, run with -Z macro-backtrace for more info)
 " "
 error[E0599]: the method `anyhow_kind` exists for reference `&Error`, but its trait bounds were not satisfied
-   --> src/main.rs:7:13
-    |
-4   | struct Error;
-    | -------------
-    | |
-    | doesn't satisfy `Error: Into<anyhow::Error>`
-    | doesn't satisfy `Error: anyhow::private::kind::TraitKind`
-    | doesn't satisfy `Error: std::fmt::Display`
+ --> src/main.rs:7:13
+  |
+4 | struct Error;
+  | -------------
+  | |
+  | doesn't satisfy `Error: Into<anyhow::Error>`
+  | doesn't satisfy `Error: anyhow::private::kind::TraitKind`
+  | doesn't satisfy `Error: std::fmt::Display`
 ...
-7   |     let _ = anyhow!(Error);
-    |             ^^^^^^^^^^^^^^ method cannot be called on `&Error` due to unsatisfied trait bounds
-    |
-    = note: the following trait bounds were not satisfied:
-            `Error: Into<anyhow::Error>`
-            which is required by `Error: anyhow::private::kind::TraitKind`
-            `Error: std::fmt::Display`
-            which is required by `&Error: anyhow::private::kind::AdhocKind`
-            `&Error: Into<anyhow::Error>`
-            which is required by `&Error: anyhow::private::kind::TraitKind`
+7 |     let _ = anyhow!(Error);
+  |             ^^^^^^^^^^^^^^ method cannot be called on `&Error` due to unsatisfied trait bounds
+  |
+  = note: the following trait bounds were not satisfied:
+          `Error: Into<anyhow::Error>`
+          which is required by `Error: anyhow::private::kind::TraitKind`
+          `Error: std::fmt::Display`
+          which is required by `&Error: anyhow::private::kind::AdhocKind`
+          `&Error: Into<anyhow::Error>`
+          which is required by `&Error: anyhow::private::kind::TraitKind`
 note: the following traits must be implemented
-   --> $RUST/core/src/convert/mod.rs
-    |
-    | / pub trait Into<T>: Sized {
-    | |     /// Performs the conversion.
-    | |     #[stable(feature = \"rust1\", since = \"1.0.0\")]
-    | |     fn into(self) -> T;
-    | | }
-    | |_^
-    |
-   ::: $RUST/core/src/fmt/mod.rs
-    |
-    | / pub trait Display {
-    | |     /// Formats the value using the given formatter.
-    | |     ///
-    | |     /// # Examples
-...   |
-    | |     fn fmt(&self, f: &mut Formatter<'_>) -> Result;
-    | | }
-    | |_^
-    = note: this error originates in the macro `anyhow` (in Nightly builds, run with -Z macro-backtrace for more info)
+ --> $RUST/core/src/convert/mod.rs
+  |
+  | / pub trait Into<T>: Sized {
+  | |     /// Performs the conversion.
+  | |     #[stable(feature = \"rust1\", since = \"1.0.0\")]
+  | |     fn into(self) -> T;
+  | | }
+  | |_^
+  |
+ ::: $RUST/core/src/fmt/mod.rs
+  |
+  | / pub trait Display {
+  | |     /// Formats the value using the given formatter.
+  | |     ///
+  | |     /// # Examples
+... |
+  | |     fn fmt(&self, f: &mut Formatter<'_>) -> Result;
+  | | }
+  | |_^
+  = note: this error originates in the macro `anyhow` (in Nightly builds, run with -Z macro-backtrace for more info)
 "}
 
 test_normalize! {test_pyo3_url
@@ -269,3 +284,144 @@ Additional crates such as `pyo3-asyncio` can be used to integrate async Rust and
 10 | async fn async_function() {}
    | ^^^^^
 "}
+
+test_normalize! {test_dropshot_required_by
+    DIR="/git/dropshot/dropshot"
+    WORKSPACE="/git/dropshot"
+    INPUT="tests/fail/bad_endpoint4.rs"
+"
+error[E0277]: the trait bound `QueryParams: schemars::JsonSchema` is not satisfied
+   --> /git/dropshot/dropshot/tests/fail/bad_endpoint4.rs:24:14
+    |
+24  |     _params: Query<QueryParams>,
+    |              ^^^^^^^^^^^^^^^^^^ the trait `schemars::JsonSchema` is not implemented for `QueryParams`
+    |
+note: required by a bound in `dropshot::Query`
+   --> /git/dropshot/dropshot/src/handler.rs:547:48
+    |
+547 | pub struct Query<QueryType: DeserializeOwned + JsonSchema + Send + Sync> {
+    |                                                ^^^^^^^^^^ required by this bound in `dropshot::Query`
+"
+"
+error[E0277]: the trait bound `QueryParams: schemars::JsonSchema` is not satisfied
+  --> tests/fail/bad_endpoint4.rs:24:14
+   |
+24 |     _params: Query<QueryParams>,
+   |              ^^^^^^^^^^^^^^^^^^ the trait `schemars::JsonSchema` is not implemented for `QueryParams`
+   |
+note: required by a bound in `dropshot::Query`
+  --> src/handler.rs
+   |
+   | pub struct Query<QueryType: DeserializeOwned + JsonSchema + Send + Sync> {
+   |                                                ^^^^^^^^^^ required by this bound in `dropshot::Query`
+"}
+
+test_normalize! {test_uniffi_out_dir
+    DIR="/git/uniffi-rs/fixtures/uitests"
+    WORKSPACE="/git/uniffi-rs"
+    TARGET="/git/uniffi-rs/target"
+"
+error[E0277]: the trait bound `Arc<Counter>: FfiConverter` is not satisfied
+   --> /git/uniffi-rs/target/debug/build/uniffi_uitests-1a51d46aecb559a7/out/counter.uniffi.rs:160:19
+    |
+160 |             match <std::sync::Arc<Counter> as uniffi::FfiConverter>::try_lift(ptr) {
+    |                   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ the trait `FfiConverter` is not implemented for `Arc<Counter>`
+    |
+    = help: the following implementations were found:
+              <Arc<T> as FfiConverter>
+    = note: required by `try_lift`
+" "
+error[E0277]: the trait bound `Arc<Counter>: FfiConverter` is not satisfied
+ --> $OUT_DIR[uniffi_uitests]/counter.uniffi.rs
+  |
+  |             match <std::sync::Arc<Counter> as uniffi::FfiConverter>::try_lift(ptr) {
+  |                   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ the trait `FfiConverter` is not implemented for `Arc<Counter>`
+  |
+  = help: the following implementations were found:
+            <Arc<T> as FfiConverter>
+  = note: required by `try_lift`
+"}
+
+test_normalize! {test_proc_macro_panic
+    DIR="D:\\repro"
+    INPUT="tests\\ui\\nonzero_fail.rs"
+"
+error[E0080]: evaluation of constant value failed
+ --> D:\\repro\\tests\\ui\\nonzero_fail.rs:7:10
+  |
+7 | #[derive(NonZeroRepr)]
+  |          ^^^^^^^^^^^ the evaluated program panicked at 'expected non-zero discriminant expression', D:\\repro\\tests\\ui\\nonzero_fail.rs:7:10
+" "
+error[E0080]: evaluation of constant value failed
+ --> tests/ui/nonzero_fail.rs:7:10
+  |
+7 | #[derive(NonZeroRepr)]
+  |          ^^^^^^^^^^^ the evaluated program panicked at 'expected non-zero discriminant expression', $DIR/tests/ui/nonzero_fail.rs:7:10
+"}
+
+test_normalize! {test_and_n_others "
+error[E0277]: no implementation for `u8 >> &str`
+ --> src/main.rs:2:20
+  |
+2 |     let _x = 42_u8 >> \"bar\";
+  |                    ^^ no implementation for `u8 >> &str`
+  |
+  = help: the trait `Shr<&str>` is not implemented for `u8`
+  = help: the following other types implement trait `Shr<Rhs>`:
+            <&'a i128 as Shr<i128>>
+            <&'a i128 as Shr<i16>>
+            <&'a i128 as Shr<i32>>
+            <&'a i128 as Shr<i64>>
+            <&'a i128 as Shr<i8>>
+            <&'a i128 as Shr<isize>>
+            <&'a i128 as Shr<u128>>
+            <&'a i128 as Shr<u16>>
+          and 568 others
+" "
+error[E0277]: no implementation for `u8 >> &str`
+ --> src/main.rs:2:20
+  |
+2 |     let _x = 42_u8 >> \"bar\";
+  |                    ^^ no implementation for `u8 >> &str`
+  |
+  = help: the trait `Shr<&str>` is not implemented for `u8`
+  = help: the following other types implement trait `Shr<Rhs>`:
+            <&'a i128 as Shr<i128>>
+            <&'a i128 as Shr<i16>>
+            <&'a i128 as Shr<i32>>
+            <&'a i128 as Shr<i64>>
+            <&'a i128 as Shr<i8>>
+            <&'a i128 as Shr<isize>>
+            <&'a i128 as Shr<u128>>
+            <&'a i128 as Shr<u16>>
+          and $N others
+"}
+
+test_normalize! {test_long_file_names "
+error: reached the recursion limit while instantiating `test::<Cons<Cons<Cons<Cons<Cons<...>>>>>>`
+  --> src/main.rs:18:11
+   |
+18 |     _ => {test (n-1, i+1, Cons {head:2*i+1, tail:first}, Cons{head:i*i, tail:second})}
+   |           ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+   |
+note: `test` defined here
+  --> src/main.rs:16:1
+   |
+16 | fn test<T:Dot> (n:isize, i:isize, first:T, second:T) ->isize {
+   | ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+   = note: the full type name has been written to `/playground/target/debug/deps/playground-c53df771d95c66fb.c7a39e8d0dd9c781.long-type-16688711729771999621.txt`
+           the full type name has been written to `/playground/target/debug/deps/playground-c53df771d95c66fb.c7a39e8d0dd9c781.long-type-16688711729771999621.txt`
+" "
+error: reached the recursion limit while instantiating `test::<Cons<Cons<Cons<Cons<Cons<...>>>>>>`
+  --> src/main.rs:18:11
+   |
+18 |     _ => {test (n-1, i+1, Cons {head:2*i+1, tail:first}, Cons{head:i*i, tail:second})}
+   |           ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+   |
+note: `test` defined here
+  --> src/main.rs:16:1
+   |
+16 | fn test<T:Dot> (n:isize, i:isize, first:T, second:T) ->isize {
+   | ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+"
+}
